/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;



    /// <summary> Used to override the mapping of a <c>Basic</c> (whether
    /// explicit or default) property or field or <c>Id</c> property or
    /// field.
    /// <para>
    ///  May be applied to an entity that extends a mapped superclass or
    /// to an embedded field or property to override a basic mapping or id
    /// mapping defined by the mapped superclass or embeddable class (or
    /// embeddable class of one of its attributes).
    ///</para>
    ///<para>
    ///  May be applied to an element collection containing instances of
    /// an embeddable class or to a map collection whose key and/or value
    /// is an embeddable class. When <c>AttributeOverride</c> is
    /// applied to a map, "<c>key.</c>" or "<c>value.</c>" must
    /// be used to prefix the name of the attribute that is being
    /// overridden in order to specify it as part of the map key or map
    /// value.
    /// </para>
    ///<para>
    ///  To override mappings at multiple levels of embedding, a dot (".")
    /// notation form must be used in the <c>name</c> element to indicate an
    /// attribute within an embedded attribute.  The value of each identifier
    /// used with the dot notation is the name of the respective embedded
    /// field or property.
    /// </para>
    ///<para>
    ///  If <c>AttributeOverride</c> is not specified, the column
    /// is mapped the same as in the original mapping.
    /// </para>
    /// <example><code>
    ///    Example 1:
    /// 
    ///    &#91;MappedSuperclass&#93;
    ///    public class Employee {
    ///        &#91;Id&#93; protected Integer id;
    ///        &#91;Version&#93; protected Integer version;
    ///        protected String address;
    ///        public Integer getId() { ... }
    ///        public void setId(Integer id) { ... }
    ///        public String getAddress() { ... }
    ///        public void setAddress(String address) { ... }
    ///    }
    /// 
    ///    &#91;Entity&#93;
    ///    &#91;AttributeOverride(PropertyName="address", Name="ADDR"))&#93;
    ///    public class PartTimeEmployee extends Employee {
    ///        // address field mapping overridden to ADDR
    ///        protected Float wage();
    ///        public Float getHourlyWage() { ... }
    ///        public void setHourlyWage(Float wage) { ... }
    ///    }
    /// 
    /// 
    ///    Example 2:
    /// 
    ///    &#91;Embeddable&#93; public class Address {
    ///        protected String street;
    ///        protected String city;
    ///        protected String state;
    ///        &#91;Embedded&#93; protected Zipcode zipcode;
    ///    }
    /// 
    ///    &#91;Embeddable&#93; public class Zipcode {
    ///        protected String zip;
    ///        protected String plusFour;
    ///    }
    /// 
    ///    &#91;Entity&#93; public class Customer {
    ///        &#91;Id&#93; protected Integer id;
    ///        protected String name;
    ///        &#91;AttributeOverride(PropertyName="state", Name="ADDR_STATE")&#93;
    ///        &#91;AttributeOverride(PropertyName="zipcode.zip", Name="ADDR_ZIP")&#93;
    ///        })
    ///        &#91;Embedded&#93; protected Address address;
    ///        ...
    ///    }
    /// 
    /// 
    ///    Example 3:
    /// 
    ///    &#91;Entity&#93; public class PropertyRecord {
    ///        &#91;Embedded&#93;Id PropertyOwner owner;
    ///        &#91;AttributeOverride(PropertyName="key.street", Name="STREET_NAME")&#93;
    ///        &#91;AttributeOverride(PropertyName="value.size", Name="SQUARE_FEET")&#93;
    ///        &#91;AttributeOverride(PropertyName="value.tax", Name="ASSESSMENT")&#93;
    ///        
    ///       &#91;ElementCollection&#93;
    ///       Dictionary&#060;Address, PropertyInfo&#062; parcels;
    ///    }
    /// 
    ///   &#91;Embeddable&#93; public class PropertyInfo {
    ///       Integer parcelNumber;
    ///       Integer size;
    ///       BigDecimal tax;
    ///   }
    /// 
    /// </code></example>
    /// 
    /// <seealso cref="Embedded"/>
    /// <seealso cref="Embeddable"/>
    /// <seealso cref="MappedSuperclass"/>
    /// <seealso cref="AssociationOverride"/>
    /// 
    /// 
    /// </summary>
[System.AttributeUsage(System.AttributeTargets.Class | System.AttributeTargets.Field | System.AttributeTargets.Property, AllowMultiple=true)] 


public sealed class AttributeOverride : Column {

    /// <summary> (Required) The name of the property whose mapping is being 
    /// overridden if property-based access is being used, or the 
    /// name of the field if field-based access is used.
    /// </summary>
    public string PropertyName;

}
}