/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    /// <summary> Specifies a mapping to an entity that is a map key. 
    /// <para>
    /// The map key
    /// join column is in the collection table, join table, or table of the
    /// target entity that is used to represent the map.  If no
    /// <c>MapKeyJoinColumn</c> annotation is specified, a single
    /// join column is assumed and the default values apply.
    /// </para>
    /// <example><code>
    /// 
    ///    Example 1:
    /// 
    ///    &#91;Entity&#93;
    ///    public class Company {
    ///       &#91;Id&#93; int id;
    ///       ...
    ///       &#91;OneToMany&#93;   // unidirectional
    ///       &#91;JoinTable(name="COMPANY_ORGANIZATION")&#93;
    ///                  &#91;JoinColumn(AttributeRef="COMPANY_ORGANIZATION",name="COMPANY")&#93;
    ///                  &#91;JoinColumn(AttributeRef="COMPANY_ORGANIZATION",name="VICEPRESIDENT", Inverse=true)&#93;
    ///       &#91;MapKeyJoinColumn(name="DIVISION")&#93;
    ///       Dictionary&#060;Division, VicePresident&#062; organization;
    ///    }
    /// 
    ///    Example 2:
    /// 
    ///    &#91;Entity&#93;
    ///    public class VideoStore {
    ///       &#91;Id&#93; int id;
    ///       String name;
    ///       Address location;
    ///       ...
    ///       &#91;ElementCollection&#93;
    ///       &#91;CollectionTable(name="INVENTORY")&#93;
    ///                  &#91;JoinColumn(AttributeRef="INVENTORY",name="STORE")&#93;
    ///       &#91;Column(name="COPIES_IN_STOCK")&#93;
    ///       &#91;MapKeyJoinColumn(name="MOVIE", referencedColumnName="ID")&#93;
    ///       Dictionary&#060;Movie, Integer&#062; videoInventory;
    ///       ...
    ///     }
    /// 
    ///     &#91;Entity&#93;
    ///     public class Movie {
    ///        &#91;Id&#93; long id;
    ///        String title;
    ///        ...
    ///     }
    /// 
    ///     Example 3:
    /// 
    ///     &#91;Entity&#93;
    ///     public class Student {
    ///        &#91;Id&#93; int studentId;
    ///        ...
    ///        &#91;ManyToMany&#93;  // students and courses are also many-many
    ///        &#91;JoinTable(name="ENROLLMENTS")&#93;
    ///                   &#91;JoinColumn(AttributeRef="ENROLLMENTS",name="STUDENT")&#93;
    ///                   &#91;JoinColumn(AttributeRef="ENROLLMENTS",name="SEMESTER", Inverse=true)&#93;
    ///        &#91;MapKeyJoinColumn(name="COURSE")&#93;
    ///        Dictionary&#060;Course, Semester&#062;  enrollment;
    ///        ...
    ///     }
    /// </code></example>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property, AllowMultiple=true)]/*@Target( { METHOD, FIELD })*/

public sealed class MapKeyJoinColumn : System.Attribute {
	/// <summary> (Optional) The name of the foreign key column for the map
	/// key. The table in which it is found depends upon the
	/// context.  
        /// <ul> 
        /// <li> If the join is for a map key for an
	/// element collection, the foreign key column is in the
    /// collection table for the map value.  </li>
        /// <li> If the join is  for a map key for a ManyToMany entity
        /// relationship or for a  OneToMany entity relationship 
    /// using a join table, the foreign key column is in a join table. </li>
        /// <li> If the join is for a  OneToMany entity relationship using 
        /// a foreign key mapping strategy, the foreign key column for the 
    /// map key is in the table of the entity that is the value of the map.</li>
        /// </ul>
        /// 
        ///  Default (only applies if a single join column is used.)
        /// The concatenation of the following: the name of the
        /// referencing relationship property or field of the
        /// referencing entity or embeddable class; "_"; "KEY".
	/// </summary>
	public string Name = "";

	/// <summary> (Optional) The name of the column referenced by this foreign key column.
	/// The referenced column is in the table of the target entity.
        /// 
        ///  Default (only applies if single join column is being
        /// used.) The same name as the primary key column of the
        /// referenced table
	/// </summary>
	public string ReferencedColumnName = "";

	/// <summary> (Optional) Whether the property is a unique key. This is a
	/// shortcut for the <c>UniqueConstraint</c> annotation
	/// at the table level and is useful for when the unique key
	/// constraint is only a single field.
	/// </summary>
	public bool Unique = false;

	/// <summary> (Optional) Whether the foreign key column is nullable.
	/// </summary>
	public bool Nullable = false;

	/// <summary> (Optional) Whether the column is included in SQL INSERT statements
	/// generated by the persistence provider.
	/// </summary>
	public bool Insertable = true;

	/// <summary> (Optional) Whether the column is included in SQL UPDATE statements
	/// generated by the persistence provider.
	/// </summary>
	public bool Updatable = true;

	/// <summary> (Optional) The SQL fragment that is used when generating the DDL for the
	/// column.
        ///  Defaults to SQL generated by the provider for the column.
	/// </summary>
	public string ColumnDefinition = "";

	/// <summary> (Optional) The name of the table that contains the foreign key column. 
        /// <ul>
        /// <li> If the join is for a map key for an element collection, the foreign key
    /// column is in the collection table for the map value. </li>
        /// <li> If the join is for a map key for a ManyToMany entity relationship 
        /// or for a OneToMany entity relationship using a join table, 
    /// the foreign key column is in a join table.</li>
	/// <li> If the join is for a OneToMany entity relationship using a foreign
	/// key mapping strategy, the foreign key column for the map key is in the
    /// table of the entity that is the value of the map.</li>
        /// </ul>
        ///  Default: 
        /// <ul>
        /// <li> If the map is for an element collection, the
    /// name of the collection table for the map value.</li>
        /// <li> If the map is for a OneToMany or ManyToMany entity relationship 
    /// using a join table, the name of the join table for the map. </li>
        /// <li> If the map is for a OneToMany entity relationship using a
        /// foreign key mapping strategy, the name of the primary table
    /// of the entity that is the value of the map.</li>
        /// </ul>
	/// </summary>
	public string Table = "";
}
}