/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;




    /// <summary> Defines a many-valued association with one-to-many multiplicity.
    /// <para>
    ///  If the collection is defined using generics to specify the 
    /// element type, the associated target entity type need not be 
    /// specified; otherwise the target entity class must be specified.
    /// If the relationship is bidirectional, the
    /// <c> mappedBy</c> element must be used to specify the relationship field or
    /// property of the entity that is the owner of the relationship.
    /// </para><para>
    ///  The <c>OneToMany</c> annotation may be used within an embeddable class
    /// contained within an entity class to specify a relationship to a
    /// collection of entities. If the relationship is bidirectional, the
    /// <c> mappedBy</c> element must be used to specify the relationship field or
    /// property of the entity that is the owner of the relationship.
    /// </para><para>
    /// When the collection is a <c>System.Collections.Generic.Dictionary</c>, the <c>cascade</c> 
    /// element and the <c>orphanRemoval</c> element apply to the map value.
    /// </para>
    /// <example><code>
    /// 
    ///    Example 1: One-to-Many association using generics
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;OneToMany&#93;(cascade=ALL, mappedBy="cUSTOMER")&#93;
    ///    public Set&#060;Order&#062; getOrders() { return orders; }
    /// 
    ///    In Order class:
    /// 
    ///    &#91;ManyToOne&#93;
    ///    &#91;JoinColumn(name="CUST_ID", nullable=false)&#93;
    ///    public Customer getCustomer() { return customer; }
    /// 
    /// 
    ///    Example 2: One-to-Many association without using generics
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;OneToMany(targetEntity=com.acme.Order.class, cascade=ALL,
    ///                mappedBy="cUSTOMER")&#93;
    ///    public Set getOrders() { return orders; }
    /// 
    ///    // In Order class:
    /// 
    ///    &#91;ManyToOne&#93;
    ///    &#91;JoinColumn(name="CUST_ID", nullable=false)&#93;
    ///    public Customer getCustomer() { return customer; }
    /// 
    /// 
    ///    Example 3: Unidirectional One-to-Many association using a foreign key mapping
    /// 
    ///    // In Customer class:
    /// 
    ///    &#91;OneToMany(orphanRemoval=true)&#93;
    ///    &#91;JoinColumn(name="CUST_ID")&#93; // join column is in table for Order
    ///    public Set&#060;Order&#062; getOrders() {return orders;}
    ///    
    /// </code></example>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)] 


public sealed class OneToMany : System.Attribute {

    /// <summary> (Optional) The entity class that is the target
    /// of the association. Optional only if the collection
    /// property is defined using generics.
    /// Must be specified otherwise.
    /// 
    ///  Defaults to the parameterized type of
    /// the collection when defined using generics.
    /// </summary>
    public System.Type TargetEntity = typeof(void);

    /// <summary> 
    /// (Optional) The operations that must be cascaded to 
    /// the target of the association.
    ///  Defaults to no operations being cascaded.
    /// 
    ///  When the target collection is a Dictionary, the <c>cascade</c> element applies to the
    /// Dictionary value.
    /// </summary>
    public CascadeType[] Cascade = {};

    /// <summary> (Optional) Whether the association should be lazily loaded or
    /// must be eagerly fetched. The EAGER strategy is a requirement on
    /// the persistence provider runtime that the associated entities
    /// must be eagerly fetched.  The LAZY strategy is a hint to the
    /// persistence provider runtime.
    /// </summary>
    public FetchType Fetch = FetchType.LAZY;

    /// <summary> 
    /// The field that owns the relationship. Required unless 
    /// the relationship is unidirectional.
    /// </summary>
    public string MappedBy = "";

    /// <summary> (Optional) Whether to apply the remove operation to entities that have
    /// been removed from the relationship and to cascade the remove operation to
    /// those entities.
    /// 
    /// </summary>
    public bool OrphanRemoval = false;
    
}
}