/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


/// <summary> Interface used to control the execution of typed queries.
    /// <para>
    /// <typeparam name ="X"> query result type</typeparam>
    /// </para>
/// <seealso cref="Query"/>
/// <seealso cref="T:NPersistence.Parameter"/>
/// 
/// 
/// </summary>
public interface TypedQuery<X> : Query {
	
    /// <summary> Execute a SELECT query and return the query results
    /// as a typed List.
    /// <returns> a list of the results</returns>
    /// <exception cref="InvalidOperationException"> if called for a Java
    ///         Persistence query language UPDATE or DELETE statement</exception>
    /// <exception cref="QueryTimeoutException"> if the query execution exceeds
    ///         the query timeout value set and only the statement is
    ///         rolled back</exception>
    /// <exception cref="TransactionRequiredException"> if a lock mode has
    ///         been set and there is no transaction</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking
    ///         fails and only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if the query execution exceeds 
    ///         the query timeout value set and the transaction 
    ///         is rolled back </exception>
    /// </summary>
    new IList<X> GetResultList();

    /// <summary> Execute a SELECT query that returns a single result.
    /// <returns> the result</returns>
    /// <exception cref="NoResultException"> if there is no result</exception>
    /// <exception cref="NonUniqueResultException"> if more than one result</exception>
    /// <exception cref="InvalidOperationException"> if called for a Java
    ///         Persistence query language UPDATE or DELETE statement</exception>
    /// <exception cref="QueryTimeoutException"> if the query execution exceeds
    ///         the query timeout value set and only the statement is
    ///         rolled back</exception>
    /// <exception cref="TransactionRequiredException"> if a lock mode has
    ///         been set and there is no transaction</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking
    ///         fails and only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if the query execution exceeds 
    ///         the query timeout value set and the transaction 
    ///         is rolled back </exception>
    /// </summary>
    new X GetSingleResult();

    /// <summary> Set the maximum number of results to retrieve.
    /// <param name="maxResult">  maximum number of results to retrieve</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the argument is negative</exception>
    /// </summary>
    new TypedQuery<X> SetMaxResults(int maxResult);

    /// <summary> Set the position of the first result to retrieve.
    /// <param name="startPosition"> position of the first result, </param>
    ///        numbered from 0
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the argument is negative</exception>
    /// </summary>
    new TypedQuery<X> SetFirstResult(int startPosition);

    /// <summary> Set a query property or hint. The hints elements may be used 
    /// to specify query properties and hints. Properties defined by
    /// this specification must be observed by the provider. 
    /// Vendor-specific hints that are not recognized by a provider
    /// must be silently ignored. Portable applications should not
    /// rely on the standard timeout hint. Depending on the database
    /// in use and the locking mechanisms used by the provider,
    /// this hint may or may not be observed.
    /// <param name="hintName">  name of property or hint</param>
    /// <param name="value">  value for the property or hint</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the second argument is not
    ///         valid for the implementation</exception>
    /// </summary>
    new TypedQuery<X> SetHint(string hintName, object value);

    /// <summary> Bind the value of a <c>Parameter</c> object.
    /// <param name="param">  parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter
    ///         does not correspond to a parameter of the
    ///         query</exception>
    /// </summary>
    new TypedQuery<X> SetParameter<T>(Parameter<T> param, T value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a <c>Parameter</c> object.
    /// <param name="param">  parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter does not
    ///         correspond to a parameter of the query</exception>
    /// </summary>
    //TypedQuery<X> SetParameter(Parameter<DateTime> param, 
    //                           DateTime value,  
    //                           TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a <c>Parameter</c> object.
    /// <param name="param">  parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter does not
    ///         correspond to a parameter of the query</exception>
    /// </summary>
    new TypedQuery<X> SetParameter(Parameter<System.DateTime> param, System.DateTime value,  
                               TemporalType temporalType);

    /// <summary> Bind an argument to a named parameter.
    /// <param name="name">  parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does 
    ///         not correspond to a parameter of the query or if
    ///         the argument is of incorrect type</exception>
    /// </summary>
    new TypedQuery<X> SetParameter(string name, object value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a named parameter.
    /// <param name="name">  parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does
    ///         not correspond to a parameter of the query or if 
    ///         the value argument is of incorrect type</exception>
    /// </summary>
    //TypedQuery<X> SetParameter(string name, DateTime value, 
    //                           TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a named parameter.
    /// <param name="name">   parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does
    ///         not correspond to a parameter of the query or if 
    ///         the value argument is of incorrect type</exception>
    /// </summary>
    new TypedQuery<X> SetParameter(string name, System.DateTime value, 
                               TemporalType temporalType);

    /// <summary> Bind an argument to a positional parameter.
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the
    ///         query or if the argument is of incorrect type</exception>
    /// </summary>
    new TypedQuery<X> SetParameter(int position, object value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a positional
    /// parameter.
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the query
    ///         or if the value argument is of incorrect type</exception>
    /// </summary>
    //TypedQuery<X> SetParameter(int position, DateTime value,  
    //                           TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a positional parameter.
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the query
    ///         or if the value argument is of incorrect type</exception>
    /// </summary>
    new TypedQuery<X> SetParameter(int position, System.DateTime value,  
                               TemporalType temporalType);

     /// <summary>
     /// Set the flush mode type to be used for the query execution.
     /// The flush mode type applies to the query regardless of the
     /// flush mode type in use for the entity manager.
     /// <param name="flushMode">  flush mode</param>
     /// <returns> the same query instance</returns>
     /// </summary>
    new TypedQuery<X> SetFlushMode(FlushModeType flushMode);

     /// <summary>
     /// Set the lock mode type to be used for the query execution.
     /// <param name="lockMode">  lock mode</param>
     /// <returns> the same query instance</returns>
     /// <exception cref="InvalidOperationException"> if the query is found not to 
     ///         be a Java Persistence query language SELECT query
    ///         or a Criteria API query</exception>
     /// </summary>
    new TypedQuery<X> SetLockMode(LockModeType lockMode);

}
}